package com.nisum.basicFlowExample;

import java.util.concurrent.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.Supplier;

public class MyPublisher extends SubmissionPublisher<Integer> {

    private static final String LOG_MESSAGE_FORMAT = "Publisher >> [%s] %s%n";
    private final int MAX_ITEM_TO_PUBLISH = 5;

    private final ScheduledFuture<?> periodicTask;
    private final ScheduledExecutorService scheduler;

    private final AtomicInteger i;

    MyPublisher(Executor executor, int maxBufferCapacity, long period, TimeUnit unit) {

        super(executor, maxBufferCapacity);

        // if using the default, normally the ForkJoinPool.commonPool(), call:
        // super();
        i = new AtomicInteger(0);

        scheduler = new ScheduledThreadPoolExecutor(1);
        periodicTask = scheduler.scheduleAtFixedRate(() -> {
            Integer item = supplier.get();

            System.out.println("publishing item: " + item + " ...");

            submit(item);

            System.out.println("estimateMaximumLag: " + super.estimateMaximumLag());
            System.out.println("estimateMinimumDemand: " + super.estimateMinimumDemand());

            if (item == MAX_ITEM_TO_PUBLISH) {
                close();
            }

        }, 0, period, unit);
    }
    Supplier<? extends Integer> supplier = new Supplier<>() {

        @Override
        public Integer get() {

            int value = i.incrementAndGet();

            return Integer.valueOf(value);
        }
    };



}

